// VString Implentation File
//

#include <iostream.h>
#include <string.h>
#include "VString.h"


// Constructor
VString::VString (const char *s)
{
	m_length = strlen(s);
	m_string = new char [m_length + 1];
	strcpy (m_string, s);
}


// Destructor
VString::~VString ()
{
	delete [] m_string;  // in pair with "new"
}


// Function Display
void VString::Display(void) const
{
	cout << m_string;
}


// Display string with prefix and suffix
void VString::Display (const char *prefix, const char *suffix, BOOL needCRLF) const
{
	cout << prefix << m_string << suffix;

	if (needCRLF) cout << '\n';
}


// assigning a character string
VString &VString::operator= (const char *other)
{		
	if (m_string)
	  delete [] m_string;

	m_length = strlen(other);
	m_string = new char [m_length + 1];
	strcpy (m_string, other);

	return *this;
}


// Assignment Operator to another String object
VString &VString::operator= (const VString &other)
{	
	if (this == &other)				// "other" is reference to another of VString object
	  return *this;						// take care of self-assignment such as object R=R

	if (m_string) delete [] m_string;				// in case of non-existence of "m_string"

	m_length = other.m_length;						  // assignment to new string length
	char *temp = new char [m_length + 1];   // create dynamic memory allocation
	strcpy (temp, other.m_string);
	m_string = temp;												// switch pointer	

	return *this;
}


// overloading an equality operator with another "object"
VString::operator== (const VString &other) const
{
	// return (strcmp(m_string, other.m_string) == 0 ? TRUE : FALSE);
	
	if (strcmp(m_string, other.m_string) == 0)
		return TRUE;
	else
		return FALSE;
}


// overloading equality operator with another "string"
BOOL VString::operator== (const char *other) const
{
	// return (strcmp(m_string, other.m_string) == 0 ? TRUE : FALSE);
	
	if (strcmp(m_string, other) == 0)
		return TRUE;
	else
		return FALSE;
}


// Function Put character
void VString::Put_At (const char Character, const int Index)
{
   m_string[Index-1] = Character;
}


// Get function
char VString::Get_At (const int Index) const
{
	return m_string[Index-1];
}


// Copy Operator Overloading
VString::VString(const VString &other)
{
	// delete of "m_string" is not neceesary
	m_length = other.m_length;
	m_string =  new char [m_length + 1];		// no need for char *m_string because it 
																					// is already defined in class/private
	strcpy (m_string, other.m_string);
}


// Append function with another VString object
void VString::Append (const VString &other)
{
	m_length += other.m_length;
	char *temp = new char [m_length + 1];

	strcpy (temp, m_string);
	strcat (temp, other.m_string);

	delete [] m_string;

	m_string = temp;
}


// Append function with another string
void VString::Append (const char *other)
{
	m_length += strlen(other);
	char *temp = new char [m_length + 1];

	strcpy (temp, m_string);
	strcat (temp, other);

	delete [] m_string;

	m_string = temp;
}


// Overloading Concatenation operator with another string
void VString::operator+= (const char *other)
{
	m_length += strlen(other);
	char *temp = new char [m_length + 1];

	strcpy (temp, m_string);
	strcat (temp, other);

	delete [] m_string;

	m_string = temp;
}


// Overloading Concatenation operator with another VString object
void VString::operator+= (const VString &other)
{
	m_length += other.m_length;
	char *temp = new char [m_length + 1];

	strcpy (temp, m_string);
	strcat (temp, other.m_string);

	delete [] m_string;

	m_string = temp;
}


// function to display substrings
void VString::Display_Substring (const int Start, const int End) const
{
	for (int i = Start; i <= End; i++) 	
	{
		cout << m_string[i-1];
	}
}


// Is_Empty function
BOOL VString::Is_Empty(void) const
{
	if (!m_string)
		return TRUE;
	else
		return FALSE;
} 


// return the length of the string object
BOOL VString::Length() const
{
	return m_length;
}


// overloading ! operator
BOOL VString::operator! (void) const
{
	return (m_length==0);  // returning TURE = empty
}

